<?php
/*
Plugin Name: File Download Tracker
Description: Un plug-in per contare quante volte un file è stato scaricato.
Version: 2.0
Author: Dario per TecnologiaDigitale
*/

if (!defined('ABSPATH')) {
    exit; // Evita l'accesso diretto ai file
}

// Funzione per inviare i dati di installazione
function fdt_track_plugin_installation() {
    $site_url = get_site_url();
    $admin_email = get_option('admin_email');
    $plugin = 'File Download Tracker';
    $version = '2.0';

    $response = wp_remote_post('https://www.tecnologiadigitale.net/track-plugin-installation.php', array(
        'method'      => 'POST',
        'body'        => array(
            'site_url'    => $site_url,
            'admin_email' => $admin_email,
            'plugin'      => $plugin,
            'version'     => $version,
        ),
    ));

    if (is_wp_error($response)) {
        error_log('Error tracking plugin installation: ' . $response->get_error_message());
    }
}

// Funzione per creare la tabella al momento dell'attivazione del plugin
function fdt_create_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'file_downloads';
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        file_name VARCHAR(255) NOT NULL,
        file_url VARCHAR(255) NOT NULL,
        download_count bigint(20) DEFAULT 0 NOT NULL,
        PRIMARY KEY (id)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}

// Funzione per creare la cartella downloads
function fdt_create_downloads_folder() {
    $upload_dir = wp_upload_dir();
    $downloads_dir = $upload_dir['basedir'] . '/downloads';

    if (!file_exists($downloads_dir)) {
        wp_mkdir_p($downloads_dir);
    }
}

// Funzione di attivazione del plugin
function fdt_activate_plugin() {
    fdt_create_table();
    fdt_create_downloads_folder();
    fdt_track_plugin_installation(); // Traccia l'installazione del plugin
}
register_activation_hook(__FILE__, 'fdt_activate_plugin');

// Funzione per tracciare i download
function fdt_track_download($file_name) {
    global $wpdb;
    $table_name = $wpdb->prefix . 'file_downloads';

    $file = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE file_name = %s", $file_name));

    if ($file) {
        $wpdb->update(
            $table_name,
            array('download_count' => $file->download_count + 1),
            array('file_name' => $file_name)
        );
    } else {
        $file_url = wp_upload_dir()['baseurl'] . '/downloads/' . $file_name;
        $wpdb->insert(
            $table_name,
            array(
                'file_name' => $file_name,
                'file_url' => $file_url,
                'download_count' => 1
            )
        );
    }

    // Aggiungi log per monitorare
    error_log('Tracciamento download per il file: ' . $file_name);
}

// Funzione per ottenere il conteggio dei download
function fdt_get_download_count($file_name) {
    global $wpdb;
    $table_name = $wpdb->prefix . 'file_downloads';

    $file = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE file_name = %s", $file_name));
    if ($file) {
        return $file->download_count;
    }
    return 0;
}

// Aggiungi lo script JavaScript
function fdt_enqueue_scripts() {
    wp_enqueue_script('fdt-script', plugins_url('/fdt-script.js', __FILE__), array('jquery'), null, true);
    wp_localize_script('fdt-script', 'fdt_ajax_object', array(
        'ajax_url' => admin_url('admin-ajax.php')
    ));
}
add_action('wp_enqueue_scripts', 'fdt_enqueue_scripts');

// Gestione delle richieste AJAX
function fdt_ajax_handler() {
    if (isset($_POST['file_name'])) {
        $file_name = sanitize_text_field($_POST['file_name']);
        fdt_track_download($file_name);
        wp_send_json_success(array('download_count' => fdt_get_download_count($file_name)));
    } else {
        wp_send_json_error();
    }
}
add_action('wp_ajax_fdt_track_download', 'fdt_ajax_handler');
add_action('wp_ajax_nopriv_fdt_track_download', 'fdt_ajax_handler');

// Shortcode per [download-track file="nome_del_file"]
function fdt_download_shortcode($atts) {
    $atts = shortcode_atts(array(
        'file' => '',
        'label' => 'Download'
    ), $atts, 'download-track');

    $file_name = sanitize_text_field($atts['file']);
    $label = esc_html($atts['label']);

    if (!$file_name) {
        return 'Nome del file mancante';
    }

    $file_url = wp_upload_dir()['baseurl'] . '/downloads/' . $file_name;

    return '<a href="' . esc_url($file_url) . '" class="download-link">' . $label . '</a>';
}
add_shortcode('download-track', 'fdt_download_shortcode');

// Aggiungi un menu di amministrazione
function fdt_add_admin_menu() {
    add_menu_page(
        'File Download Tracker',
        'Download Tracker',
        'manage_options',
        'file-download-tracker',
        'fdt_display_download_counts',
        'dashicons-download',
        20
    );

    add_submenu_page(
        'file-download-tracker',
        'Carica File',
        'Carica File',
        'manage_options',
        'fdt-upload-file',
        'fdt_display_upload_form'
    );

    add_submenu_page(
        'file-download-tracker',
        'Istruzioni Plugin',
        'Istruzioni Plugin',
        'manage_options',
        'fdt_instructions',
        'fdt_display_instructions_page'
    );
}
add_action('admin_menu', 'fdt_add_admin_menu');

// Funzione per visualizzare le statistiche di download
function fdt_display_download_counts() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'file_downloads';
    $uploads_dir = wp_upload_dir()['basedir'] . '/downloads';
    $files = array_diff(scandir($uploads_dir), array('.', '..'));

    echo '<div class="wrap">';
    echo '<h1>Statistiche di Download</h1>';
    echo '<table class="wp-list-table widefat fixed striped">';
    echo '<thead>';
    echo '<tr>';
    echo '<th>Nome del File</th>';
    echo '<th style="width: 400px;">Shortcode</th>'; // Allunghiamo la cella per lo shortcode
    echo '<th>Conteggio Download</th>';
    echo '<th>Azioni</th>';
    echo '</tr>';
    echo '</thead>';
    echo '<tbody>';

    if (!empty($files)) {
        foreach ($files as $file) {
            $file_name = basename($file);
            $shortcode = '[download-track file="' . $file_name . '"]';

            // Recupera il conteggio dei download se il file è tracciato
            $download_count = 0;
            $tracked_file = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name WHERE file_name = %s", $file_name));
            if ($tracked_file) {
                $download_count = $tracked_file->download_count;
            }

            echo '<tr>';
            echo '<td>' . $file_name . '</td>';
            echo '<td><input type="text" class="shortcode-input" value="' . esc_attr($shortcode) . '" readonly="readonly" style="width: 100%;"></td>';
            echo '<td>' . $download_count . '</td>';
            echo '<td><a href="' . admin_url('admin.php?page=file-download-tracker&delete_file=' . urlencode($file_name)) . '" class="button button-danger">Elimina</a></td>';
            echo '</tr>';
        }
    } else {
        echo '<tr><td colspan="4">Nessun file trovato nella cartella dei download.</td></tr>';
    }

    echo '</tbody>';
    echo '</table>';
    echo '</div>';
}

// Funzione per visualizzare il form di caricamento file
function fdt_display_upload_form() {
    echo '<div class="wrap">';
    echo '<h1>Carica un Nuovo File</h1>';
    echo '<form method="post" enctype="multipart/form-data">';
    echo '<input type="file" name="fdt_file" />';
    echo '<input type="submit" name="fdt_upload_file" value="Carica File" class="button button-primary" />';
    echo '</form>';
    echo '</div>';

    if (isset($_POST['fdt_upload_file'])) {
        fdt_handle_file_upload();
    }
}

// Funzione per gestire il caricamento del file
function fdt_handle_file_upload() {
    if (!empty($_FILES['fdt_file']['name'])) {
        $uploaded = wp_handle_upload($_FILES['fdt_file'], array('test_form' => false));

        if ($uploaded && !isset($uploaded['error'])) {
            $upload_dir = wp_upload_dir();
            $downloads_dir = $upload_dir['basedir'] . '/downloads';

            $file_name = basename($uploaded['file']);
            $destination = $downloads_dir . '/' . $file_name;

            // Verifica se il file esiste già e sovrascriverlo
            if (file_exists($destination)) {
                unlink($destination);
            }

            if (rename($uploaded['file'], $destination)) {
                echo '<div class="updated"><p>File caricato con successo!</p></div>';
            } else {
                echo '<div class="error"><p>Errore nel spostare il file.</p></div>';
            }
        } else {
            echo '<div class="error"><p>Errore nel caricamento del file: ' . $uploaded['error'] . '</p></div>';
        }
    } else {
        echo '<div class="error"><p>Per favore seleziona un file da caricare.</p></div>';
    }
}

// Funzione per visualizzare la pagina delle istruzioni
function fdt_display_instructions_page() {
    echo '<div class="wrap">';
    echo '<h1>Istruzioni del Plugin</h1>';
    echo '<p>Per utilizzare questo plugin, segui i seguenti passaggi:</p>';
    echo '<ol>';
    echo '<li>Carica i file nella cartella <code>wp-content/uploads/downloads</code> usando la sezione "Carica File".</li>';
    echo '<li>Inserisci lo shortcode <code>[download-track file="nome_del_file"]</code> negli articoli o nelle pagine dove desideri inserire il link di download.</li>';
    echo '<li>Puoi visualizzare le statistiche di download nella pagina "Download Tracker" del menu di amministrazione.</li>';
    echo '</ol>';
    echo '</div>';
}

// Funzione per eliminare un file
function fdt_delete_file() {
    if (isset($_GET['delete_file'])) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'file_downloads';
        $file_name = sanitize_text_field($_GET['delete_file']);

        $upload_dir = wp_upload_dir();
        $file_path = $upload_dir['basedir'] . '/downloads/' . $file_name;

        if (file_exists($file_path)) {
            unlink($file_path);
        }

        // Rimuovi il record dal database se esiste
        $wpdb->delete($table_name, array('file_name' => $file_name));
        echo '<div class="updated"><p>File eliminato con successo.</p></div>';
    }
}
add_action('admin_notices', 'fdt_delete_file');

// Funzione di disinstallazione del plugin
function fdt_uninstall_plugin() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'file_downloads';

    // Elimina la tabella
    $wpdb->query("DROP TABLE IF EXISTS $table_name");

    // Elimina la cartella dei download
    $upload_dir = wp_upload_dir();
    $downloads_dir = $upload_dir['basedir'] . '/downloads';
    if (file_exists($downloads_dir)) {
        array_map('unlink', glob("$downloads_dir/*.*"));
        rmdir($downloads_dir);
    }
}
register_uninstall_hook(__FILE__, 'fdt_uninstall_plugin');
?>
